#include "application.h"

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>


Application *Application::applicationList;
Application::Option *Application::Option::constructionList;


//--------------------------------------------------
// Main - starts an application
//--------------------------------------------------

int main(int argc, char *argv[])
{
  Application *app=Application::applicationList;

  if((!app) || app->next)
    {
      fprintf(stderr,app?"Error: multiple applications defined.\n"
	                :"Error: no applications defined.\n");
      assert(0);
      return 0;
    }

  if(app->parseOptions(argc,argv))
    {
      return app->main();
    }

  app->printHelp();
  
  return 0;
}


//--------------------------------------------------
// Application::Option
//--------------------------------------------------

Application::Option::Option()
{
  next=constructionList;constructionList=this;
}


Application::Option *Application::Option::getOptionList()
{
  Option *ret=constructionList;constructionList=0;return ret;
}


//--------------------------------------------------
// Application::StringMathingOption
//--------------------------------------------------

bool Application::StringMatchingOption::isExactMatch(const char *s)
{
  return std::string(s)==std::string(matchString);
}


Application::StringMatchingOption::StringMatchingOption(const char *s):
  matchString(s)
{
}


bool Application::StringMatchingOption::matches(const char *s)
{
  for(int i=0;matchString[i];i++)
    if(matchString[i]!=s[i])return false;
  return true;
}


//--------------------------------------------------
// Application::SimpleOption
//--------------------------------------------------

Application::SimpleOption::SimpleOption(const char *s):
  StringMatchingOption(s),
  value(false)
{
}


void Application::SimpleOption::parseValue(int t, char **argv, bool *ok, int *numberOfArgumentsParsed)
{
  *numberOfArgumentsParsed=0;
  if(isExactMatch(argv[t]))
    {
      value=true;
      *ok=true;
    }
  else
    *ok=false;
}


bool Application::SimpleOption::getValue()
{
  return value;
}


//--------------------------------------------------
// Application::ValueOption
//--------------------------------------------------

Application::ValueOption::ValueOption(const char *s):
  StringMatchingOption(s)
{
}


void Application::ValueOption::parseValue(int t, char **argv, bool *ok, int *numberOfArgumentsParsed)
{
  if(isExactMatch(argv[t]))
    {
      if(argv[t+1]==0 || argv[t+1][0]=='-')
        {
          assignValue("");
          *ok=true;
          *numberOfArgumentsParsed=0;
        }
      else
        {
          assignValue(argv[t+1]);
          if(argv[t+1])
            {
              *ok=true;
              *numberOfArgumentsParsed=1;
            }
          else
            {
              *ok=false;
              *numberOfArgumentsParsed=0;
            }
	}
    }
  else
    {
      assignValue(argv[t]+std::string(matchString).length());
      *ok=true;
      *numberOfArgumentsParsed=0;
    }
}


//--------------------------------------------------
// Application::StringOption
//--------------------------------------------------

Application::StringOption::StringOption(const char *s, const char *initialValue):
  ValueOption(s),
  value(initialValue)
{
}


void Application::StringOption::assignValue(const char *s)
{
  value=s;
}


const char *Application::StringOption::getValue()
{
  return value;
}


//--------------------------------------------------
// Application::IntegerOption
//--------------------------------------------------

Application::IntegerOption::IntegerOption(const char *s, int initialValue):
  ValueOption(s),
  value(initialValue),
  hasRange(false)
{
}


Application::IntegerOption::IntegerOption(const char *s, int initialValue, int lower_, int upper_):
  ValueOption(s),
  value(initialValue),
  hasRange(true),
  lower(lower_),
  upper(upper_)
{
}


void Application::IntegerOption::assignValue(const char *s)
{
  bool onlyDigits=true;
  value=0;
  for(int t=0;s[t];t++)
    {
      if(s[t]<'0' || s[t]>'9')onlyDigits=false;
      value*=10;
      value+=s[0]-'0';
    }
  assert(onlyDigits);

  if(hasRange)
    if(lower>value || upper<value)
      {
	fprintf(stderr,"Integer option %s has out of range value. Range={%i,...,%i}\n",matchString,lower,upper);
	assert(0);
      }
}


int Application::IntegerOption::getValue()
{
  return value;
}


//--------------------------------------------------
// Application::ZeroOneOption
//--------------------------------------------------

Application::ZeroOneOption::ZeroOneOption(const char *s, int initialValue):
  IntegerOption(s,initialValue,0,1)
{
}


//--------------------------------------------------
// Application
//--------------------------------------------------

Application::Application()
{
  next=applicationList;applicationList=this;optionList=Option::getOptionList();
}


bool Application::parseOptions(int argc, char **argv)
{
  for(int t=1;t<argc;t++)
    {
      int numberOfMatchesFound=0;
      
      for(Option *i=optionList;i;i=i->next)
	{
	  if(i->matches(argv[t]))numberOfMatchesFound++;
	}
      //      fprintf(stderr,"NumberOfMatches %i\n",numberOfMatchesFound);
      assert(numberOfMatchesFound<2);
      if(numberOfMatchesFound==0)return false;
      
      for(Option *i=optionList;i;i=i->next)
	if(i->matches(argv[t]))
	  {
	    bool ok=false;
	    int argumentsParsed=1;
	    i->parseValue(t,argv,&ok,&argumentsParsed);
	    if(!ok)
	      {
		fprintf(stderr,"Parse error parsing arguments in option %s\n",argv[t]);
		assert(0);
		exit(0);
	      }
	    t+=argumentsParsed;
	    break;
	  }
    }
  return true;
}


void Application::registerOptions()
{//merge current list of options with new list
  Option *l=Option::getOptionList();
  while(l)
    {
      Option *o=l;
      l=o->next;
      o->next=optionList;
      optionList=o;
    }
}


void Application::printHelp()
{
  fprintf(stderr,"%s",helpText());
};


//--------------------------------------------------
// CatsApplication
//--------------------------------------------------

void CatsApplication::printHelp()
{
  fprintf(stderr,"This program is a part of the CaTS software package for computing initial ideals of toric ideals. See the readme file for more information or visit the CaTS homepage at http://www.soopadoopa.dk/anders/cats.html.\n\n");
  Application::printHelp();
};
