#include <algorithm>

#include "fiber.h"
#include "graver.h"
#include "toric.h"
#include "parser.h"
#include "printer.h"
#include "lll.h"
#include "application.h"

struct CompareClass
{
  bool lex(vektor_i const &a, vektor_i const &b) const
  {
    assert(a.size()==b.size());
    for(int i=0;i<a.size();i++)
      {
	if(a[i]>b[i])return false;
	if(a[i]<b[i])return true;
      }
    return false;
  }

  bool operator()(vektor_i const &a, vektor_i const &b) const
  {
    int sa=a.lt().sum();
    int sb=b.lt().sum();
    if(sa<sb)return true;
    if(sa>sb)return false;
    return lex(a.lt(),b.lt());
  }
};


class HighDegreeFibersApplication : public CatsApplication
{
  const char *helpText()
  {
    return "Given a graded matrix A this program will compute the Graver basis using 4ti2\n"
      "For each Graver fiber of degree less than rank of A it will print the fiber.\n"
      "At the end of the output the Graver basis is divided in two sets.\n"
      "One with low degree and one with high degree.\n"
      "It will also tell if any high degree Graver fiber has only two elements.\n"
      "(Conjecture: which is not the case if A is normal.\n"
      "The input should be a matrix in CaTS' format.\n";
  }

  int main()
  {
    gbasis input=FileParser(stdin).parseMatrix();
    basis_i A=toIndexedMatrix(input);
    
    gbasis g=toric1(toIndexedMatrix(input));
    
    GraverBasis graverBasis(toIndexedMatrix(input));
    
    gbasis graverList;
    for(GraverBasis::iterator i=graverBasis.begin();i!=graverBasis.end();i++)graverList.push_back(*i);
    
    graverList.sort(CompareClass());
    printf("Number of elements in Graver basis:%i\n",graverList.size());
    set<vektor_i,CompareClass> degreeSet;
    
    int rankA=rank(A);
    printf("Matrix A:");
    AsciiPrinter(stdout).printMatrix(A);
    printf("Rank = %i\n",rankA);
    
    gbasis graverHigh;
    gbasis graverLow;
    bool hasSmallHighDegreeFiber=false;
    
    for(gbasis::iterator i=graverList.begin();i!=graverList.end();i++)
      {
	int d=i->lt().sum();
	
	if(d<=rankA)
	  graverLow.push_back(*i);
	else
	  graverHigh.push_back(*i);
	vektor_i degree=multiply(A,i->lt());
	if(degreeSet.count(degree)==0)
	  {
	  fprintf(stdout,"---------------------------------------------------------\n");
	  degreeSet.insert(degree);
	  if(d<=rankA)
	    {
 	      printf("Graver binomial with low degree (%i<=%i).\n",d,rankA);
	      printf("Multidegree: ");
	      AsciiPrinter(stdout).printVector(degree);
	      printf("\n");
	    }
	  else
	    {
 	      printf("Graver binomial with high degree (%i>%i).\n",d,rankA);
	      printf("Multidegree: ");
	      AsciiPrinter(stdout).printVector(degree);
	      printf("\n");
	    }

	  fprintf(stdout,"Graver binomials:\n{\n");
	  bool first=true;
	  for(GraverBasis::iterator j=graverBasis.begin();j!=graverBasis.end();++j)
	    {
	      if(degree==multiply(A,j->lt()))
		{
		  if(!first)
		    fprintf(stdout,",\n");
		  AsciiPrinter(stdout).printBinomial(*j);
		  first=false;
		}
	    }
	  fprintf(stdout,"\n}\n");

	  Fiber f(g,i->lt());
	  fprintf(stdout,"Fiber:\n");
	  AsciiPrinter p(stdout);
	  f.print(p);

	  int fiberSize=0;
	  for(Fiber::Iterator j=f.begin();j!=f.end();++j)fiberSize++;

	  if(d>rankA && fiberSize==2)hasSmallHighDegreeFiber=true;
	}
      }
    
    fprintf(stdout,"Low degree binomials:\n");
    AsciiPrinter(stdout).printGroebnerBasisNewLine(graverLow);
    fprintf(stdout,"High degree binomials:\n");
    AsciiPrinter(stdout).printGroebnerBasisNewLine(graverHigh);

    fprintf(stdout,hasSmallHighDegreeFiber? "A high degree fiber has only two elements!!!\n":"All high degree fibers have more than two elements.\n");
    return 0;
  }
};


static HighDegreeFibersApplication theApplication;
