#include "normaliz.h"
#include "parser.h"
#include "printer.h"
#include "facet.h"
#include "toric.h"
#include "radical.h"
#include "macaulay2.h"
#include "lp.h"

#include "application.h"


static Macaulay2Pipe *M2;

class InteractiveApplication : public CatsApplication
{
  const char *helpText()
  {
    return "This program takes a point configuration\n"
      "or a Groebner basis as input and lets you flip its facets interactively.\n"
      "The point configuration / Groebner basis should be given on the standard\n"
      "input in CaTS' format. If a point configuration is given the starting\n"
      "reduced Groebner basis will be computed using the reverse lexicographic\n"
      "order with a>b>c>d...\n"
      "In the interactive mode you can choose which facet binomial you want to\n"
      "flip by typing in its index.\n"
      "See the CaTS README file for more information.\n"
      "IMPORTANT: If you allow non-geometric flips and you supply your own\n"
      "starting Groebner basis this must be a true Groebner basis.\n"
      "Options:\n"
      "   -i filename  select a file instead of the stdin as input for the first\n"
      "                Groebner basis.\n"
      "   -A           allow all A-graded flips. That is allowing flips using non-\n"
      "                geometric facets.\n";
  }

  SimpleOption optionAGraded;
  StringOption optionInputFile;

  gbasis trueGroebnerBasis;

  gbasis myFlip(const gbasis& g, const vektor_i& p)
  {
    if(optionAGraded.getValue())
      return flipAGraded(g,p,trueGroebnerBasis);
    return flip(g,p);
  }

  void interactiveWalk(gbasis old, gbasis current)
  {
    static char line[256];
    
    sortLex(&old);
    sortLex(&current);
    
    bool ok=true;
    
    while(1)
      {
	gbasis facets=optionAGraded.getValue()?algebraicFacets(current):irredundant(current);

	printf("-----------------------------------------\n");
	printf("%i : %i\n",algebraicFacets(current).size(),irredundant(current).size());
	printf("Current Groebner basis:\n");
	AsciiPrinter(stdout).printGroebnerBasis(current);
	printf("\nRadical:");
	AsciiPrinter(stdout).printMonomialList(initialradical(current));
	printf("\nStandardPairs:\n");
	StandardPairList spl=computeStandardPairs(current);
	AsciiPrinter(stdout).printStandardPairList(spl);
	printf("Degree: %i\n",degree(current));
	printf("Has embedded primes: %s\n",hasEmbeddedPrimes(current)?"true":"false");
	
	printf("\nFacets:\n");
	int n=0;
	for(gbasis::const_iterator i=facets.begin();i!=facets.end();i++)
	  {
	    printf("%2i: ",n++);
	    AsciiPrinter(stdout).printBinomial(*i);
	    
	    {
	      gbasis n=myFlip(current,*i);
	      gbasisminimal(&n);
	      gbasisautoreduction(&n);
	      sortLex(&n);
	      
	      gbcmp cmp;
	      if(cmp(n,old)||cmp(old,n))printf("   ");else printf("---");
	      gbasis r1=initialradical(current);
	      gbasis r2=initialradical(n);
	      StandardPairList spl2=computeStandardPairs(n);
	      if(spl2.size()>spl.size())printf("(%i > %i)",spl2.size(),spl.size());
	      else
		if(spl2.size()<spl.size())printf("(%i < %i)",spl2.size(),spl.size());
	      //	    if(((cmp(r1,r2))) || ((cmp(r2,r1))))printf("  to new radical");
	      if(flipsToOtherRadical(r1,*i))printf("  to new radical");
	      if(!isfacet(facets,i))printf("  non-geometric facet");
	    }
	    printf("\n");
	  }
	fprintf(stdout,"?");
	fscanf(stdin,"%s",line);
	
	switch(line[0])
	  {
	  case 'b':
	    return;
	  case 'c':
	    if(!M2)M2=new Macaulay2Pipe();
	    M2->setPolynomialRing(current);
	    {
	    gbasis initial=initialIdeal(current);
	    printf("pdim coker gens: %i",M2->getPdimCokerGensMonomial(initial));
	    }
	    delete M2;
	    M2=0;
	    break;
	  default:
	    if(line[0]>='0' && line[0]<='9')
	      {
		int num;
		sscanf(line,"%i",&num);
	      
		gbasis::iterator i=facets.begin();
		while(num-->0 && i!=facets.end())i++;
		if(i==facets.end())ok=false;
		else
		  {
		    gbasis n=myFlip(current,*i);
		    
		    gbasisminimal(&n);
		    gbasisautoreduction(&n);
		    interactiveWalk(current,n);
		  }
	      }
	    else ok=false;
	  }
	if(!ok)fprintf(stderr,"error\n");
      }
  }
 
public:
  InteractiveApplication():
    optionAGraded("-A"),
    optionInputFile("-i")
  {
    registerOptions();
  }

  int main()
  {
    FILE *f=stdin;
    if(optionInputFile.getValue())
      {
	f=fopen(optionInputFile.getValue(),"r");
	assert(f);
      }
    
    basis_i A;
    gbasis inputgbasis;
    int inputtype=FileParser(f).parseInput(&A,&inputgbasis);
    
    if(optionInputFile.getValue())fclose(f);
    gbasis g;
    switch(inputtype)
      {
      case 1:
	g=toric1(A);
	break;
      case 2:
	g=inputgbasis;
	break;
      default:
	assert(0);
      }
    
    trueGroebnerBasis=g;
    
    interactiveWalk(g,g);
    
    return 0;
  }
};

static InteractiveApplication theApplication;
